""" Python objects representing common powerpoint shapes """

#############
## Imports ##
#############

from .base import Object, Group
from .rectangle import Rectangle
from .line import Line
from .text import Text
from .canvas import Canvas
from .polygon import Polygon
from .image import Image
from .image import Mesh
from .spine import Spine
